﻿using System;
using System.Configuration;
using System.Runtime.CompilerServices;

namespace ConfigurationTest
{
    /// <summary>
    /// 配置类，可以获取或设置指定的配置项
    /// </summary>
    public abstract class ConfigSetting
    {
        /// <summary>
        /// 配置类
        /// </summary>
        /// <param name="configuration">配置</param>
        public ConfigSetting(Configuration configuration)
        {
            Configuration = configuration;
        }

        /// <summary>
        /// 当前配置
        /// </summary>
        public Configuration Configuration
        {
            get;
        }

        /// <summary>
        /// 获取当前程序配置
        /// </summary>
        /// <param name="config"></param>
        /// <returns></returns>
        public static Configuration GetCurrentConfiguration()
        {
            return ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
        }

        /// <summary>
        /// 返回指定的配置项的值
        /// </summary>
        /// <param name="settingKey"></param>
        /// <returns></returns>
        public string GetSettingValueByKey(string settingKey)
        {
            return GetSettingValue(settingKey);
        }

        /// <summary>
        /// 返回指定的配置项的值
        /// </summary>
        /// <param name="settingKey"></param>
        /// <returns></returns>
        public T GetSettingValueByKey<T>(string settingKey)
        {
            return GetSettingValue<T>(settingKey);
        }

        /// <summary>
        /// 为指定的配置项设置值
        /// </summary>
        /// <param name="value"></param>
        /// <param name="settingKey"></param>
        public void SetSettingValueByKey(string settingKey, object value)
        {
            SetSettingValue(value, settingKey);
        }

        /// <summary>
        /// 返回指定的配置项的值
        /// </summary>
        /// <param name="settingKey"></param>
        /// <returns></returns>
        protected virtual string GetSettingValue([CallerMemberName] string settingKey = null)
        {
            return Configuration?.AppSettings?.Settings[settingKey]?.Value;
        }

        /// <summary>
        /// 返回指定的配置项的值
        /// </summary>
        /// <typeparam name="T">值类型</typeparam>
        /// <param name="settingKey"></param>
        /// <returns></returns>
        protected virtual T GetSettingValue<T>([CallerMemberName] string settingKey = null)
        {
            var value = GetSettingValue(settingKey);

            if (string.IsNullOrWhiteSpace(value))
            {
                return default(T);
            }
            else
            {
                return (T)Convert.ChangeType(value, typeof(T));
            }
        }

        /// <summary>
        /// 为指定的配置项设置值
        /// </summary>
        /// <param name="value"></param>
        /// <param name="settingKey"></param>
        protected virtual void SetSettingValue(object value, [CallerMemberName] string settingKey = null)
        {
            Configuration.AddOrUpdateAppSettingItem(settingKey, value?.ToString());
            Configuration.Save();
        }
    }
}